@@ -2,8 +2,10 @@ from collections import defaultdict
 from concurrent.futures.thread import ThreadPoolExecutor
 from csv import DictReader
 from datetime import datetime, timedelta
+from functools import reduce
 from typing import Union
 from urllib.parse import urlsplit, quote_plus
+from libinsitu._version import __version__
 
 import numpy as np
 from dateutil.relativedelta import relativedelta
@@ -144,6 +146,11 @@ def parseCSV(res_path, key = "ID") :
     res = dict()
     rows = DictReader(read_res(res_path))
     for row in rows:
+
+        # Skip commented lines
+        if "#" in row[key] :
+            continue
+
         res[row[key]] = {key: parse_value(val) for key, val in row.items()}
     return res
 
@@ -176,7 +183,7 @@ def getStationInfo(network, station_id) :
 def getNetworkInfo(network) :
     networks = getNetworksInfo()
     if not network in networks :
-        raise Exception("Network %s not found in Network infp of %s" % network)
+        raise Exception("Network %s not found in Network info" % network)
     return networks[network]
 
 def is_uniform(vector) :
@@ -383,6 +390,14 @@ def match_pattern(pattern, value, properties=dict()) :
 
     return res
 
+def get_df_resolution(df) :
+    """Get resolution of a time series Dataframe in seconds. Either from metdata or from guessing it"""
+    if GLOBAL_TIME_RESOLUTION_ATTR in df.attrs :
+        return df.attrs[GLOBAL_TIME_RESOLUTION_ATTR]
+
+    # Guess it from data
+    return df.index.to_series().diff().median().total_seconds()
+
 def netcdf_to_dataframe(
         ncfile : Union[Dataset, str],
         start_time: Union[datetime, datetime64]=None,
@@ -398,7 +413,8 @@ def netcdf_to_dataframe(
         chunked=False,
         chunk_size=CHUNK_SIZE,
         steps=1,
-        rename_cols=False) :
+        rename_cols=False,
+        expand_qc=False):
     """
         Load NETCDF in-situ file (or part of it) into a panda Dataframe, with time as index.
 
@@ -406,7 +422,16 @@ def netcdf_to_dataframe(
         :param ncfile: NetCDF Dataset or filename, or OpenDAP URL
         :param rename_cols: If True (default) rename solar irradiance columns as per convention (GHI, BNI, DHI)
         :param drop_duplicates: If true (default), duplicate rows are droppped
-        :param skip_qc: If True, skip lines with bad QC (at least one failing)
+        :param skip_qc:
+
+            If true, filters rows having any failing QC. False by default (no filter).
+
+            You can also provide a list of flags to filter : `["T3C_bsrn_3cmp", "T2C_seri_kn_kt"]`
+
+            Or filter on any flags but some, by prepending '!' : `["!T3C_bsrn_3cmp", "!T2C_seri_kn_kt"]`
+
+            For full list of flags, see the [online doc](https://libinsitu.readthedocs.io/en/latest/qc.html)
+
         :param skip_na: If True, drop rows containing only nan values
         :param start_time: Start time (first record by default) : Datetime or datetime64
         :param end_time: End time (last record by default) : Datetile or datetime64
@@ -418,7 +443,9 @@ def netcdf_to_dataframe(
         :param chunked: If True, does not load the whole file in memory at once : returns an iterator on Dataframe chunks.
         :param chunk_size: Size of chunks for chunked data
         :param steps: Downsampling (1 by default)
-        :return: Pandas Dataframe, or iterator on Dataframes is chunk is activated
+        :param expand_qc: If True, expand the QC bitmaps into one boolean column for each flag with name "QC.<flag>"
+
+        :return: Pandas Dataframe, or iterator on Dataframes if chunking is activated
         """
 
     chunks = __nc2df(
@@ -436,7 +463,8 @@ def netcdf_to_dataframe(
         chunk_size=chunk_size,
         steps=steps,
         rename=rename_cols,
-        skip_qc=skip_qc)
+        skip_qc=skip_qc,
+        expand_qc=expand_qc)
 
     # Handling either single result or chunked generator
     if not chunked :
@@ -474,6 +502,53 @@ def __all_attributes(ncfile) :
 
     return attrs
 
+def _skip_qc_to_mask(df, flags) :
+
+    # 32 ones bitmap
+    ones_mask = 0xffffffff
+    if flags is True:
+        return ones_mask
+    if not flags :
+        return 0
+
+    # At this point, flags is a list of flags or negative (!) flags
+
+    # Extract (!)
+    neg = [flag.startswith("!") for flag in flags]
+    flags = list(flag.replace("!", "") for flag in flags)
+
+    # Ensure not mixed negative and positive flags
+    if neg[1:] != neg[:-1] :
+        raise Exception("You cannot mix positive and negative (!) flags")
+
+    masks = qc_masks(df)
+
+    if neg[0]:
+        # Negative flags
+        return reduce(
+            lambda a, b : a & ~b,
+            list(masks[flag] for flag in flags),
+            ones_mask)
+    else:
+        return reduce(
+            lambda a, b: a | b,
+            list(masks[flag] for flag in flags), 0)
+
+
+
+def _expand_qc(df) :
+
+    # Get bitmaps
+    bitmaps = df[QC_FLAGS_VAR]
+
+    #  Create column applying mask for each one
+    return {
+        "QC.%s" % col : (bitmaps & mask > 0).astype(int)
+        for col, mask in qc_masks(df).items()}
+
+
+
+
 def __nc2df(
         ncfile : Union[Dataset, str],
         start_time: Union[datetime, datetime64]=None,
@@ -488,7 +563,9 @@ def __nc2df(
         password=None,
         chunked=False,
         chunk_size=CHUNK_SIZE,
-        steps=1, rename=True) :
+        steps=1, rename=True,
+        expand_qc=False) :
+
     """Private generator use by nc2df """
 
     if isinstance(ncfile, str) :
@@ -545,7 +622,15 @@ def __nc2df(
             df = df.dropna(axis=0, how='all', subset=subset)
 
         if skip_qc and QC_FLAGS_VAR in df.columns :
-            df = df[df[QC_FLAGS_VAR] == 0]
+            qc_mask = _skip_qc_to_mask(df, skip_qc)
+            df = df[(df[QC_FLAGS_VAR] & qc_mask) == 0]
+
+        if expand_qc and QC_FLAGS_VAR in df.columns :
+            flags = _expand_qc(df)
+            del df[QC_FLAGS_VAR]
+
+            for col, values in flags.items() :
+                df[col] = values
 
         # Rename variables
         if rename :
@@ -555,7 +640,6 @@ def __nc2df(
                         warning("Renaming %s -> %s" % (source, dest))
                         df = df.rename(columns={source:dest})
 
-
         return df
 
     if chunked :
@@ -682,21 +766,33 @@ def getNetworkId(attributes_or_ncfile) :
 def getStationId(attributes_or_ncfile) :
     return getMult(attributes_or_ncfile, STATION_ID_ATTRS)
 
+def _prepare_properties(
+        network_properties,
+        station_properties) :
+    """Prefix properties with Network_ and Station_, and add 'live' properties """
+
+    res = dict(
+        **{STATION_PREFIX + k: v for k, v in station_properties.items()},
+        **{NETWORK_PREFIX + k: v for k, v in network_properties.items()})
+
+    # Add live properties
+    now = datetime.now().isoformat()
+
+    res["UpdateTime"] = now
+    res["CreationTime"] = now
+    res["Version"] = __version__
+    return res
+
+
 def getProperties(network_id, station_id) :
     """Gather Network_ and Station_ properties """
 
-    # Get properties for this station
-    properties = {STATION_PREFIX + k : v for k, v in getStationInfo(network_id, station_id).items()}
-
-    # Add properties of this network
-    for key, val in getNetworkInfo(network_id).items():
-        properties[NETWORK_PREFIX + key] = val
-
-    return properties
-
+    return _prepare_properties(
+        getNetworkInfo(network_id),
+        getStationInfo(network_id, station_id))
 
 def qc_masks(df) :
-    """Parse metadata of a QC bitmap and returns dict of meaning => mask"""
+    """Parse metadata of a QC bitmap and returns dict of flag name => mask"""
     attrs = df.attrs["variables"][QC_FLAGS_VAR]
     return {meaning: mask for meaning, mask in zip(
         attrs["flag_meanings"].split(),
